import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ChessMove, ComputerConfguration, StockfishQueryParams, StockfishResponse } from './models';
import { BehaviorSubject, Observable, of, switchMap } from 'rxjs';
import { Color, FENChar } from '../../chess-logic/models';

@Injectable({
  providedIn: 'root',

})
export class StockfishService {

  // private readonly api = 'https://stockfish.online/api/s/v2.php';
  private readonly api = 'http://192.168.1.70:31701/getBestMove';

  public computerConfiguration$ = new BehaviorSubject<ComputerConfguration>({ color: Color.Black, level: 1 });

  constructor(private http: HttpClient) { }

  private convertColumnLetterToYCoord = (string: string) => string.charCodeAt(0) - "a".charCodeAt(0);

  private promotedPiece(piece: string | undefined): FENChar | null {
    if (piece === undefined) return null;
    const computerColor = this.computerConfiguration$.value.color;
    if (computerColor === Color.White) {
      switch (piece.toLowerCase()) {
        case 'q': return FENChar.WhiteQueen;
        case 'r': return FENChar.WhiteRook;
        case 'b': return FENChar.WhiteBishop;
        case 'n': return FENChar.WhiteKnight;
        default: return null;
      }
    }
    else {
      switch (piece.toLowerCase()) {
        case 'q': return FENChar.BlackQueen;
        case 'r': return FENChar.BlackRook;
        case 'b': return FENChar.BlackBishop;
        case 'n': return FENChar.BlackKnight;
        default: return null;
      }
    }
  }

  private moveFromStockfishString(move: string): ChessMove {
    console.log('moveFromStockfishString: Got move', move);
    const prevY: number = this.convertColumnLetterToYCoord(move[0]);
    const prevX: number = Number(move[1]) - 1;
    const newY: number = this.convertColumnLetterToYCoord(move[2]);
    const newX: number = Number(move[3]) - 1;
    const promotedPiece = this.promotedPiece(move[4]);
    const newChessMove = { prevX, prevY, newX, newY, promotedPiece };
    return newChessMove;
  }
  public getBestMove(fen: string): Observable<ChessMove> {
    const queryParams: StockfishQueryParams = {
      fen,
      depth: this.computerConfiguration$.value.level
    };

    return this.http.post<StockfishResponse>(this.api, queryParams)
      .pipe(
        switchMap((response: StockfishResponse) => {
          const bestmove: string = response.bestmove.split(' ')[1];
          return of(this.moveFromStockfishString(bestmove));
        })
      )
  }
}
